import venv
import os
import entry_points_txt

import subprocess
from subprocess import CompletedProcess

from pathlib import Path

from typing import Sequence

from .exceptions import VEnvOutputError



class VEnv:
    """A Python virtual environment"""

    def __init__(self, path: str, **kwargs):
        self.path = Path(path)
        venv.create(self.path, **kwargs)


    # |--------------------
    # | TERMINAL COMMANDS

    def python(self, *args: Sequence[str]) -> CompletedProcess:
        """Run Python"""
        return subprocess.run(
            [self.path / 'bin' / 'python', *args],
            capture_output=True, text=True
        )

    def run_package(self, package_name: str, *args: Sequence[str]) -> CompletedProcess:
        """Run an installed Python package"""
        return self.python('-m', package_name, *args)

    def pip(self, *args: Sequence[str]) -> CompletedProcess:
        """Run PIP"""
        return self.run_package('pip', *args)


    # |-----------
    # | METADATA

    def installed_packages(self) -> list[str]:
        """Get installed packages"""

        # Get installed packages from `pip list`
        pkgs_string = self.pip('list').stdout
        packages = [line.split(maxsplit=1)  # "package_name  0.1.0"
                        for line in pkgs_string.splitlines()]

        # "Package" and "Version" headings are expected from a valid output
        if len(packages) < 1 or tuple(packages[0]) != ('Package', 'Version'):
            raise VEnvOutputError('`python -m pip list` did not return a valid list of packages')

        # Ignore the headings and divider and return the list
        return packages[2:]

    def entry_points(self) -> dict[dict[str, str]]:
        """Get entry points"""
        
        found = {}  # Found entry points

        # Go through all files in the `lib` directory
        for root, _, files in os.walk(self.path / 'lib', topdown=False):
            if 'entry_points.txt' not in files:
                continue

            # Open and parse every `entry_points.txt`
            with open(Path(root) / 'entry_points.txt', 'r') as f:
                for name, entries in entry_points_txt.load(f).items():
                    if name not in found:
                        found[name] = {}
                    
                    # Add entry points!
                    found[name].update(entries)

        return found


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['venvarium']

package_data = \
{'': ['*']}

install_requires = \
['entry-points-txt>=0.2.0,<0.3.0']

setup_kwargs = {
    'name': 'venvarium',
    'version': '0.1.0.post1',
    'description': 'Object-based interaction with a Python virtual environment.',
    'long_description': '# venvarium\n\n## Installation\n\n```sh\npython -m pip install venvarium\n```\n\n### Example\n\n```py\nfrom venvarium import VEnv\n\n\n# Create the virtual environment\nENV_PATH = \'myproject/myenv\' \nvenv = VEnv(ENV_PATH)\n\n# Run Python, PIP, or any other package or program\nvenv.python(\'-c\', \'print("hello, world!")\')\nvenv.pip(\'install\', \'--upgrade\', \'pip\')\nvenv.run_package(\'flask\')\n\n# See all installed packages\npkgs = venv.installed_packages()\nprint(pkgs)\n\n# Get the entry points\nentry_points = venv.entry_points().get(\'my_entry_point\')\nprint(entry_points)\n```',
    'author': 'Maximillian Strand',
    'author_email': 'maximillian.strand@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/deepadmax/venvarium',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
